(***********************************************************************
 *
 * main.pas
 *
 * Copyright (c) 2016 Michel Deslierres
 * All rights reserved. Tous droits réservés.
 *
 * Read full license in the file license.txt
 * Lire la licence au complet dans le fichier license.txt
 *
 *************************************************************************)

unit Main;

{$IFDEF FPC}
  {$mode objfpc}
  {$h+ }
{$ENDIF}


interface

uses
  SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls;

type

  { TMainForm }

  TMainForm = class(TForm)
    Bevel1: TBevel;
    cbEdit: TComboBox;
    Button1: TButton;
    Label1: TLabel;
    Label2: TLabel;
    QuitButton: TButton;
    OptionsButton: TButton;
    SpeedButton1: TSpeedButton;
    procedure Button1Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure QuitButtonClick(Sender: TObject);
    procedure Button1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure OptionsButtonClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    { Déclarations privées }
    procedure ClearResult;
  public
    { Déclarations publiques }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

uses
  ParserTypes, mdMathTokenizer, mdMathEvaluator, Environment, Info, Options;

procedure TMainForm.Button1Click(Sender: TObject);
var
  value: float;
  tstart, tlen: integer;
begin
  if MathEvaluator.RaiseExceptions then begin
    label1.Caption := '';
    with MathEvaluator do
      label1.Caption := floats(EValuateExpression(cbEdit.Text));
  end
  else begin
    value := MathEvaluator.EValuateExpression(cbEdit.Text);
    if MathEvaluator.ErrorCode = meNoError then begin
      label1.Caption := MathEvaluator.floats(value);
      tstart := length(cbEdit.Text);
      tlen := 0;
    end
    else begin
      if (MathEvaluator.ErrorCode in ParserErrors) then
        label1.Caption := Format('%s, found "%s"', [MathEvaluator.ErrorMessage, MathEvaluator.Token])
      else
        label1.Caption := MathEvaluator.ErrorMessage;
      tstart := MathEvaluator.TokenStart-1;
      tlen := MathEvaluator.TokenLength;
    end;
    cbEdit.SetFocus;
    cbEdit.SelStart := tstart;
    cbEdit.SelLength := tlen;
  end;
end;

procedure TMainForm.Button4Click(Sender: TObject);
begin
  ContextInfoForm.ShowModal;
end;

procedure TMainForm.Button1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  ClearResult;
end;

procedure TMainForm.QuitButtonClick(Sender: TObject);
begin
  Close
end;

procedure TMainForm.ClearResult;
begin
  Label1.Caption := '© Copyright 2013-2016 Michel Deslierres'
end;

procedure TMainForm.Edit1Change(Sender: TObject);
begin
  ClearResult;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  {$IFDEF UNIT_INFO}
  Caption := 'DEMO: ' +  mdMathEvaluator.SVersionName + ' ' + mdMathEvaluator.SVersion;
  {$ENDIF}
  ClearResult;
  Label1.Autosize := false;
  Label1.Width := cbEdit.left + cbEdit.Width - label1.Left;
  Label1.Anchors :=  [akLeft, akTop, akRight];
end;

procedure TMainForm.OptionsButtonClick(Sender: TObject);
begin
  OptionsForm.ShowModal;
end;

procedure TMainForm.SpeedButton1Click(Sender: TObject);
begin
  if SpeedButton1.Down then
    ContextInfoForm.Show
  else
    ContextInfoForm.Hide;
end;

end.
