(***********************************************************************
 *
 * environment.pas
 *
 * Copyright (c) 2016 Michel Deslierres
 * All rights reserved. Tous droits réservés.
 *
 * Read full license in the file license.txt
 * Lire la licence au complet dans le fichier license.txt
 *
 *************************************************************************
 *
 * version 1.1
 *   Includes report on EUCLIDEAN_DIV_MOD and EXPONENT_LEFT_ASSOCIATIVE
 *   directives and unit minimath.pas.
 *
 *************************************************************************)

unit environment;

{$IFDEF FPC}
  {$mode objfpc}
  {$h+ }
{$ENDIF}

interface

uses
  SysUtils, Classes, mdMathEvaluator;

var
  MathEvaluator: TMathEvaluator;

procedure GetEnvironmentInfo(strings: TStrings);

implementation

uses
  Math, MiniMath, ParserTypes, ParserConsts, mdMathTokenizer;

const
  snoyes: array[boolean] of string = ('no', 'yes');

procedure GetEnvironmentInfo(strings: TStrings);
var
  s, ss: string;
  FPUMask: TFPUExceptionMask;

  procedure ShowContentOfFPUMask;
  const
    tab = '    ';
  begin
    if exInvalidOp in FPUMask then
      strings.add(tab + 'exInvalidOp is masked (exception disabled)')
    else
      strings.add(tab + 'exInvalidOp is not masked (exception enabled)');

    if exDenormalized in FPUMask then
      strings.add(tab + 'exDenormalized is masked (exception disabled)')
    else
      strings.add(tab + 'exDenormalized is not masked (exception enabled)');

    if exZeroDivide in FPUMask then
      strings.add(tab + 'exZeroDivide is masked (exception disabled)')
    else
      strings.add(tab + 'exZeroDivide is not masked (exception enabled)');

    if exOverflow in FPUMask then
      strings.add(tab + 'exOverflow is masked (exception disabled)')
    else
      strings.add(tab + 'exOverflow is not masked (exception enabled)');

    if exUnderflow in FPUMask then
      strings.add(tab + 'exUnderflow is masked (exception disabled)')
    else
      strings.add(tab + 'exUnderflow is not masked (exception enabled)');

    if exPrecision in FPUMask then
      strings.add(tab + 'exPrecision is masked (exception disabled)')
    else
      strings.add(tab + 'exPrecision is not masked (exception enabled)');
  end;

begin
  strings.Clear;
  {$IFDEF UNIT_INFO}
  strings.Add('Units');
  strings.Add('  ' + MiniMath.SVersionName + '  ' + MiniMath.SVersion);
  strings.Add('  ' + ParserTypes.SVersionName + '  ' + ParserTypes.SVersion);
  strings.Add('  ' + ParserConsts.SVersionName + '  ' + ParserConsts.SVersion);
  strings.Add('  ' + mdMathTokenizer.SVersionName + '  ' + mdMathTokenizer.SVersion);
  strings.Add('  ' + mdMathEvaluator.SVersionName + '  ' + mdMathEvaluator.SVersion);
  strings.Add('');
  {$ENDIF}

  if MathEvaluator.RaiseExceptions then
    strings.add('EvaluateExpression raises exceptions on errors (* can be changed)')
  else
    strings.Add('EvaluateExpression does not raise exceptions on errors (* can be changed)');

  strings.add('');
  strings.add('System decimal separator: ' + DefaultFormatSettings.DecimalSeparator );
  strings.add('Parser decimal separator: ' + MathEvaluator.DecimalSeparator + ' (* can be changed)');
  strings.add('');

  strings.add('Floating point unit exception mask');

  strings.add('');
  FPUMask := GetExceptionMask;
  strings.add('  Current FPU exception mask: $' + inttohex(dword(FPUMask), 4));
  ShowContentOfFPUMask;

  {$IFnDEF NO_EXCEPTION_MASK}
  FPUMask := MathEvaluator.ExceptionMask;
  strings.add('');
  strings.add('  Math evaluator exception mask: $' + inttohex(dword(FPUMask), 4));
  ShowContentOfFPUMask;
  {$ENDIF}

  strings.add('');
  strings.add('Parser.inc conditional defines');

  s := '  EUCLIDEAN_DIV_MOD defined: ';
  {$IFDEF EUCLIDEAN_DIV_MOD}
    ss := snoyes[true];
  {$ELSE}
    ss := snoyes[false];
  {$ENDIF}
  strings.add(s + ss);

  s := '  EXPONENT_LEFT_ASSOCIATIVE defined: ';
  {$IFDEF EXPONENT_LEFT_ASSOCIATIVE}
    ss := snoyes[true];
  {$ELSE}
    ss := snoyes[false];
  {$ENDIF}
  strings.add(s + ss);

  s := '  NO_NLS: ';
  {$IFDEF NO_NLS}
    ss := snoyes[true];
  {$ELSE}
    ss := snoyes[false];
  {$ENDIF}
  strings.add(s + ss);

  s := '  NO_EXCEPTION_MASK defined: ';
  {$IFDEF NO_EXCEPTION_MASK}
    ss := snoyes[true];
  {$ELSE}
    ss := snoyes[false];
  {$ENDIF}
  strings.add(s + ss);

  s := '  UNIT_INFO defined: ';
  {$IFDEF UNIT_INFO}
    ss := snoyes[true];
  {$ELSE}
    ss := snoyes[false];
  {$ENDIF}
  strings.add(s + ss);

  s := '  DEBUG defined: ';
  {$IFDEF DEBUG}
    ss := snoyes[true];
  {$ELSE}
    ss := snoyes[false];
  {$ENDIF}
  strings.add(s + ss);

  s := '  RELEASE defined: ';
  {$IFDEF RELEASE}
    ss := snoyes[true];
  {$ELSE}
    ss := snoyes[false];
  {$ENDIF}
  strings.add(s + ss);
end;

initialization
  MathEvaluator := TMathEvaluator.create;
finalization
  MathEvaluator.free;
end.
