(***********************************************************************
 *
 * MiniMath version 1.0
 *
 * Copyright (c) 2015 Michel Deslierres
 * All rights reserved. Tous droits réservés.
 *
 * Read full license in the file license.txt
 * Lire la licence au complet dans le fichier license.txt
 *
 *************************************************************************)
 
 
{: Unit that provides some mathematical functions not found in Math.pas }

unit MiniMath;

{$I parser.inc}

interface

uses
  parsertypes;

{$IFDEF UNIT_INFO}
const
  SVersionName = 'MiniMath';
  SVersion = '1.0';
  VersionMajor = 1;
  VersionMinor = 0;
{$ENDIF}

 {: Integer division and modulo, combined div and mod }
procedure IDivMod(const X, Y: float; var Q: Integer; var R: float);

  {: Integer modulo division }
function IMod(const X, Y: float): float;

  {: Integer division }
function IDiv(const X, Y: float): Integer;

implementation

uses
  sysutils;

(***************************************************
Reference:

Division and Modulus for Computer Scientists
DAAN LEIJEN
University of Utrecht
Dept. of Computer Science
PO.Box 80.089, 3508 TB Utrecht
The Netherlands
daan@cs.uu.nl,http://www.cs.uu.nl/~daan/lvm.html
December 3, 2001

/* Euclidean division */
long divE( long D, long d )
{
  long q = D/d;
  long r = D%d;
  if (r < 0) {
    if (d > 0) q = q-1;
          else q = q+1;
  }
  return q;
}

long modE( long D, long d )
{
  long r = D%d;
  if (r < 0) {
    if (d > 0) r = r + d;
          else r = r - d;
 }
 return r;
}

*******************************************************)

{$IFDEF EUCLIDEAN_DIV_MOD}

procedure IDivMod(const X, Y: float; var Q: Integer; var R: float);
begin
  Q := trunc(X/Y);
  R := X - Q*Y;
  if R < 0 then begin
    if Y > 0 then begin
      R := R + Y;
      dec(Q)
    end
    else begin
      R := R - Y;
      Inc(Q);
    end;
  end;
end;

function IMod(const X, Y: float): float;
var
  Q: integer;
begin
  //  DivModE(X,Y,Q, result);
  Q := trunc(X/Y);
  result := X - Q*Y;
  if result < 0 then begin
    if Y > 0 then begin
      result := result + Y;
    end
    else begin
      result := result - Y;
    end;
  end;
end;

function IDiv(const X, Y: float): Integer;
begin
  //DivModE(X,Y,result, R);
  result := trunc(X/Y);
  if X - Result*Y < 0 then begin
    if Y > 0 then begin
      dec(result)
    end
    else begin
      Inc(result);
    end;
  end;
end;

{$ELSE}
{Default Borland TP / Delphi div mod}

procedure IDivMod(const X, Y: float; var Q: Integer; var R: float);
begin
  Q := trunc(X/Y);
  R := X - Q*Y
end;

function IMod(const X, Y: float): float;
begin
  result := X - trunc(X/Y)*Y;
end;

function IDiv(const X, Y: float): Integer;
begin
  result := trunc(X/Y)
end;

{$ENDIF}


end.
