(***********************************************************************
 *
 * Parser.inc version 1.1
 *
 * Copyright (c) 2016 Michel Deslierres
 * All rights reserved. Tous droits rservs.
 *
 * Read full license in the file license.txt
 * Lire la licence au complet dans le fichier license.txt
 *
 ***********************************************************************
 *
 * This file documents the compiler directives used in Simple parser. By
 * default none are defined in this file except of UNIT_INFO.
 *
 * Changements / Changes
 *
 * - version 1.1
 * 
 *     Added EXPONENT_LEFT_ASSOCIATIVE and EUCLIDEAN_DIV_MOD
 *
 *************************************************************************)

{: File with program wide compiler defines }

{$IFNDEF PARSER_INC}
  {$DEFINE PARSER_INC}

    { If this directive is defined then the div and mod functions
      (':' and '%') will give result corresponding to the Euclidean 
      definitions of those operations. By default this directive is 
      not defined and the result returned is the same as found in Free 
      Pascal and Delphi.}
  {.DEFINE EUCLIDEAN_DIV_MOD}

    { If this directive is defined, then the expression '2^3^4' will be 
      calculated as (2^3)^4. While this may be the how LibreOffice.Calc
      and Excel do it, it seems strange. By default this directive is 
      not defined and the expression '2^3^4' is calculated as 2^(3^4).}
  {.DEFINE EXPONENT_LEFT_ASSOCIATIVE}

    { If this directive is defined then the decimal separator will be
      the period ('.') and the list separator will be the comma (',') no
      matter what the user's locale is. Furthermore the Encode and 
      Decode functions will not convert these values. By default this 
      directive is not defined and the user's locale values are used.
      In Linux/Mac OSX, clocale must be loaded in the project file.}
  {.DEFINE NO_NLS}

    { If this directive is defined, the simple parser will not change
      the FPU exception mask and the ExceptionMask property will not
      be defined. By default the directive is not defined and the FPU 
      exception mask is set to the content of the ExceptionMask before 
      any evaluation of an expression. The 'system' exceptions mask is 
      restored as soon as possible after evaluation including before 
      invoking any exception handler. This conditional must not be 
      defined if using the GTK2 widget set because it assumes that all 
      FPU exceptions are masked.}
  {.DEFINE NO_EXCEPTION_MASK}

    { If the symbol UNIT_INFO is defined then version information about units
      making up the parser is is included in each unit. If the symbol RELEASE is 
      defined, then unit version information will not be included no matter whether 
      UNIT_INFO is defined here.}
  {.DEFINE UNIT_INFO}

 {---- do not change any define below ----}

  {$IFDEF LCLGTK2}{$IFDEF NO_EXCEPTION_MASK}
    { May get away without playing with fpu exception mask, but not
      when using the the Gtk2 widget set}
    {$UNDEF NO_EXCEPTION_MASK}
  {$ENDIF}{$ENDIF}

  {$IFDEF RELEASE}
     {$UNDEF UNIT_INFO}
  {$ENDIF}
{$ENDIF}
