{: Unit used to test two versions of Div and Mod functions, the 
   actual code is included in MiniMath according to the directive
   DELPHI_DIV_MOD}

unit DivMod;

interface

  {: Euclidean division, combined div and mod }
procedure DivModE(const X, Y: extended; var Q: Integer; var R: extended);

  {: Euclidean modulo division }
function ModE(const X, Y: extended): extended;

  {: Euclidean integer division }
function DivE(const X, Y: extended): Integer;

  {: Truncated division, combined div and mod, Delphi compatible
     but defined for real operands }
procedure DivModT(const X, Y: extended; var Q: Integer; var R: extended);

  {: Truncated modulo division, Delphi compatible but defined
     for real operands }
function ModT(const X, Y: extended): extended;

  {: Truncated integer division, Delphi compatible but defined for
     real operands }
function DivT(const X, Y: extended): Integer;

implementation


(***************************************************
Reference:

Division and Modulus for Computer Scientists
DAAN LEIJEN
University of Utrecht
Dept. of Computer Science
PO.Box 80.089, 3508 TB Utrecht
The Netherlands
daan@cs.uu.nl,http://www.cs.uu.nl/~daan/lvm.html
December 3, 2001

/* Euclidean division */
long divE( long D, long d )
{
  long q = D/d;
  long r = D%d;
  if (r < 0) {
    if (d > 0) q = q-1;
          else q = q+1;
  }
  return q;
}

long modE( long D, long d )
{
  long r = D%d;
  if (r < 0) {
    if (d > 0) r = r + d;
          else r = r - d;
 }
 return r;
}

*******************************************************)

procedure DivModE(const X, Y: extended; var Q: Integer; var R: extended);
begin
  Q := trunc(X/Y);
  R := X - Q*Y;
  if R < 0 then begin
    if Y > 0 then begin
      R := R + Y;
      dec(Q)
    end
    else begin
      R := R - Y;
      Inc(Q);
    end;
  end;
end;

function ModE(const X, Y: extended): extended;
var
  Q: integer;
begin
  //  DivModE(X,Y,Q, result);
  Q := trunc(X/Y);
  result := X - Q*Y;
  if result < 0 then begin
    if Y > 0 then begin
      result := result + Y;
    end
    else begin
      result := result - Y;
    end;
  end;
end;

function DivE(const X, Y: extended): Integer;
begin
  //DivModE(X,Y,result, R);
  result := trunc(X/Y);
  if X - Result*Y < 0 then begin
    if Y > 0 then begin
      dec(result)
    end
    else begin
      Inc(result);
    end;
  end;
end;

procedure DivModT(const X, Y: extended; var Q: Integer; var R: extended);
begin
  Q := trunc(X/Y);
  R := X - Q*Y
end;

function ModT(const X, Y: extended): extended;
begin
  result := X - trunc(X/Y)*Y;
end;

function DivT(const X, Y: extended): Integer;
begin
  result := trunc(X/Y)
end;

end.
